/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

/*
 * Get the customm board name by reading efuses
 */
#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/imx-common/sys_proto.h>

/*UL board id registers */
#define OCOTP_GP34 0x021bc940
#define UL_ID_MASK 0x000FFF00
#define VESTA_100 0x00007700
#define VESTA_200 0x00008300
#define VESTA_200R 0x00008200
#define VESTA_300 0x00008100
#define VESTA_300R 0x00008000
#define VESTA_400 0x00012000
#define VESTA_RAD 0x0
#define VESTA_64M_NOR 0x1

/*ULL board id registers */
#define OCOTP_GP1 0x021bc660
#define OCOTP_GP2 0x021bc670
#define ULL_ID_MASK 0x0FFC0000
#define VESTA_300B 0x01F80000
#define VESTA_300B1 0x007c0000

DECLARE_GLOBAL_DATA_PTR;

static int do_getboard(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	unsigned int board_id;

	if (is_cpu_type(MXC_CPU_MX6ULL)) {
		board_id = (readl(OCOTP_GP1) != 0)?readl(OCOTP_GP1):readl(OCOTP_GP2);
		if (((board_id & ULL_ID_MASK) == VESTA_300B) || ((board_id & ULL_ID_MASK) == VESTA_300B1)) {
			setenv("board_type", "vesta-300B");
			setenv("fdt_file", "zImage-vesta-300B.dtb");
		}
		else {
			printf ("Board id not recognized\n");
		}
	}
	else if (is_cpu_type(MXC_CPU_MX6UL)) {
		board_id = readl(OCOTP_GP34);
		switch (board_id & UL_ID_MASK) {

			case VESTA_100 :
				setenv("board_type", "vesta-100");
				setenv("fdt_file", "zImage-vesta-100.dtb");
				break;
			case VESTA_200 :
			case VESTA_200R :
				setenv("board_type", "vesta-200");
				setenv("fdt_file", "zImage-vesta-200.dtb");
				break;
			case VESTA_300 :
			case VESTA_300R :
				setenv("board_type", "vesta-300");
				setenv("fdt_file", "zImage-vesta-300.dtb");
				break;
			case VESTA_400 :
				setenv("board_type", "vesta-400");
				setenv("fdt_file", "zImage-vesta-400.dtb");
				break;
			default:
				printf ("Board id not recognized");
				break;
		};
	}

	return 0;
}

U_BOOT_CMD(
	getboard,	2,	1,	do_getboard,
	"get board name","get board name"
);
